import pandas as pd
import torch

from factual_scene_graph.evaluation.evaluator import Evaluator
from factual_scene_graph.parser.scene_graph_parser import SceneGraphParser

device = "cuda" if torch.cuda.is_available() else "cpu"
def test_scene_graph_parsing():

    parser = SceneGraphParser('lizhuang144/flan-t5-base-VG-factual-sg', device=device)
    evaluator = Evaluator(parser=parser, device='cuda:0')
    scores = evaluator.evaluate(["2 beautiful pigs are flying on the sky with 2 bags on their backs"],[['( pigs , is , beautiful ) , ( bags , on back of , pigs ) , ( bags , is , 2 ) , ( pigs , is , 2 ) , ( pigs , fly on , sky )']],method='spice', beam_size=1, max_output_len=128)
    print(scores)

def test_scene_graph_parsing_on_random():
    parser = SceneGraphParser('lizhuang144/flan-t5-base-VG-factual-sg', device=device,lemmatize=False, lowercase=True)
    evaluator = Evaluator(parser=parser,text_encoder_checkpoint='all-MiniLM-L6-v2',  device=device,lemmatize=True)

    random_data_pd = pd.read_csv('data/factual_sg/random/test.csv')
    random_data_captions = random_data_pd['caption'].tolist()
    random_data_graphs = [[scene] for scene in random_data_pd['scene_graph'].tolist()]
    spice_scores, cand_graphs, ref_graphs = evaluator.evaluate(random_data_captions, random_data_graphs, method='spice', beam_size=1, batch_size=128, max_input_len=256, max_output_len=256, return_graphs=True)

    print('SPICE scores for random test set:')
    print(sum(spice_scores)/len(spice_scores))

    set_match_scores = evaluator.evaluate(cand_graphs, ref_graphs,method='set_match', beam_size=1)

    print('Set Match scores for random test set:')
    print(sum(set_match_scores)/len(set_match_scores))

    soft_spice_scores = evaluator.evaluate(cand_graphs, ref_graphs,method='soft_spice', beam_size=1)

    print('Soft-SPICE scores for random test set:')
    print(sum(soft_spice_scores)/len(soft_spice_scores))



if __name__ == "__main__":
    #test_scene_graph_parsing()
    test_scene_graph_parsing_on_random()


from importlib import resources

from factual_scene_graph.evaluation.synonym_dictionary import SynonymDictionary


with resources.open_text('factual_scene_graph.evaluation.resources', 'english.exceptions') as f_exc, \
     resources.open_text('factual_scene_graph.evaluation.resources', 'english.synsets') as f_syn:
    synonym_dictionary = SynonymDictionary(f_exc.name, f_syn.name)
print(synonym_dictionary.get_stem_synsets('write_down'))


import pandas as pd
import torch

from factual_scene_graph.evaluation.evaluator import Evaluator
from factual_scene_graph.parser.scene_graph_parser import SceneGraphParser

device = "cuda" if torch.cuda.is_available() else "cpu"

def test_scene_graph_parsing_on_random():
    evaluator = Evaluator(parser=None,text_encoder_checkpoint='all-MiniLM-L6-v2',  device=device,lemmatize=True)

    random_data_pd = pd.read_csv('data/factual_sg_id/random/test.csv')
    random_cand_captions = random_data_pd['caption'].tolist()


    caption_scene_dict = {}

    for line in open(file='tests/test_data/SPICE_parsing_outputs.txt', mode='r').readlines():
        caption_scene_dict[line.split('\t')[0].strip()] = line.split('\t')[1].strip()
    # print(caption_scene_dict)
    random_cand_graphs = []
    for caption in random_cand_captions:
        random_cand_graphs.append(caption_scene_dict[caption])

    random_data_graphs = [[scene] for scene in random_data_pd['scene_graph'].tolist()]
    spice_scores, cand_graphs, ref_graphs = evaluator.evaluate(random_cand_graphs, random_data_graphs, method='spice', beam_size=1, batch_size=128, max_input_len=256, max_output_len=256, return_graphs=True)

    print('SPICE scores of SPICE Parser for random test set:')
    print(sum(spice_scores)/len(spice_scores))

    set_match_scores = evaluator.evaluate(cand_graphs, ref_graphs,method='set_match', beam_size=1)

    print('Set Match scores of SPICE Parser for random test set:')
    print(sum(set_match_scores)/len(set_match_scores))

    soft_spice_scores = evaluator.evaluate(cand_graphs, ref_graphs,method='soft_spice', beam_size=1)

    print('Soft-SPICE scores of SPICE Parser for random test set:')
    print(sum(soft_spice_scores)/len(soft_spice_scores))



if __name__ == "__main__":
    #test_scene_graph_parsing()
    test_scene_graph_parsing_on_random()


from factual_scene_graph.parser.scene_graph_parser import SceneGraphParser
from factual_scene_graph.utils import space_out_symbols_in_graph, tprint, clean_graph_string


def test_clean_graph_string():
    # Test normal case
    assert clean_graph_string("( bench , is , woo") == "( bench )"
    # Test with extra spaces
    assert clean_graph_string("( bench , is , wooden ) ,") == "( bench , is , wooden )"
    # Test with multiple relations
    assert clean_graph_string("( bench , is , wooden ) , ( bench , v:faces , ") == "( bench , is , wooden )"
    # Test empty string
    assert clean_graph_string("") == ""
    print("All tests for clean_graph_string passed!")

def test_space_out_symbols_in_graph():
    # Test normal case
    assert space_out_symbols_in_graph("(bench,is,wooden)") == "( bench , is , wooden )"
    # Test with extra spaces
    assert space_out_symbols_in_graph(" (bench ,is,wooden) ") == "( bench , is , wooden )"
    # Test with multiple relations
    assert space_out_symbols_in_graph("(bench,is,wooden),(bench,v:faces,sea)") == "( bench , is , wooden ) , ( bench , v:faces , sea )"
    # Test empty string
    assert space_out_symbols_in_graph("") == ""
    print("All tests for space_out_symbols_in_graph passed!")

def test_scene_graph_parser():
    # Assuming SceneGraphParser is correctly instantiated as `parser`
    # Test normal input
    parser = SceneGraphParser('lizhuang144/flan-t5-base-VG-factual-sg-id', device='cuda:0')
    text_graph = parser.parse(["2 beautiful pigs are flying on the sky with 2 bags on their backs"],return_text=True)

    print(text_graph[0])
    text_graph = parser.parse(["2 beautiful and strong pigs are flying on the sky with 2 bags on their backs",
                              "a blue sky"], max_output_len=128, return_text=True)
    print(text_graph[0])

    graph_obj = parser.parse(["2 beautiful and strong pigs are flying on the sky with 2 bags on their backs",
                              "a blue sky"], max_output_len=128, return_text=False)
    print(graph_obj[0])

    tprint(graph_obj[0])

    graph_obj = parser.parse(["boy"], max_output_len=16, return_text=False)
    print(graph_obj[0])

    tprint(graph_obj[0])

    text_graph = parser.parse(["a logo is written on another logo"],return_text=True,filter_factual_chars=True)

    print(text_graph[0])


if __name__ == "__main__":
    test_space_out_symbols_in_graph()
    test_clean_graph_string()
    test_scene_graph_parser()


import json
import os
import pickle
import sys

import numpy as np
import scipy
import torch
from factual_scene_graph.evaluation.evaluator import Evaluator
from factual_scene_graph.parser.scene_graph_parser import SceneGraphParser


def collect_unique_captions(candidates, refs):
    """Collect unique captions from candidates and references."""
    caption_set = set(candidates)  # Add all candidates to the set
    for ref_list in refs:
        caption_set.update(ref_list)  # Add all elements from each list in refs
    return list(caption_set)

def parse_captions(captions, parser):
    """Parse captions and return a dictionary of results."""
    parse_results = parser.parse(captions, batch_size=64, max_input_len=256,
                                 max_output_len=256, beam_size=1, return_text=True)
    return {caption: result
            for caption, result in zip(captions, parse_results)}

def evaluate_graphs(candidates, refs, parse_dict, evaluator, return_graphs):
    """Evaluate the graphs and return the results."""
    cand_graphs = [parse_dict[cand] for cand in candidates]
    ref_graphs = [[parse_dict[ref_i] for ref_i in ref] for ref in refs]
    return evaluator.evaluate(cand_graphs, ref_graphs, method='spice',
                              beam_size=1, batch_size=64, max_input_len=256,
                              max_output_len=256, return_graphs=return_graphs)

def compute_correlation(input_json, tauvariant='c'):
    data = {}
    with open(input_json) as f:
        data.update(json.load(f))
    print('Loaded {} images'.format(len(data)))

    refs = []
    candidates = []
    human_scores = []
    for k, v in list(data.items()):
        for human_judgement in v['human_judgement']:
            if np.isnan(human_judgement['rating']):
                print('NaN')
                continue

            candidate = ' '.join(human_judgement['caption'].split())
            candidates.append(candidate)

            ref = [' '.join(gt.split()) for gt in v['ground_truth']]
            refs.append(ref)
            human_scores.append(human_judgement['rating'])
    print('Loaded {} references and {} candidates'.format(len(refs), len(candidates)))
    assert len(candidates) == len(refs)

    device = "cuda" if torch.cuda.is_available() else "cpu"

    parser = SceneGraphParser('lizhuang144/flan-t5-base-VG-factual-sg', device=device, lemmatize=False, lowercase=True)
    evaluator = Evaluator(parser=parser, text_encoder_checkpoint='all-MiniLM-L6-v2', device=device, lemmatize=True)


    caption_list = collect_unique_captions(candidates, refs)
    parse_dict = parse_captions(caption_list, parser)

    # Evaluate with return_graphs=True
    spice_scores, cand_graphs, ref_graphs = evaluate_graphs(candidates, refs, parse_dict, evaluator, True)


    assert len(spice_scores) == len(human_scores)
    print('SPICE score: ', sum(spice_scores) / len(spice_scores))
    print('{} Tau-{}: {:.3f}'.format(tauvariant, tauvariant, 100*scipy.stats.kendalltau(spice_scores, human_scores, variant=tauvariant)[0]))


    soft_spice_scores = evaluator.evaluate(cand_graphs, ref_graphs, method='soft_spice', batch_size=128)
    assert len(soft_spice_scores) == len(human_scores)
    print('Soft-SPICE score: ', sum(soft_spice_scores) / len(soft_spice_scores))
    print('{} Tau-{}: {:.3f}'.format(tauvariant, tauvariant, 100*scipy.stats.kendalltau(soft_spice_scores, human_scores, variant=tauvariant)[0]))



if __name__ == '__main__':
    compute_correlation('tests/test_data/flickr8k.json', tauvariant='c')


import torch
from nltk import WordNetLemmatizer
from tqdm import tqdm
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM

from ..utils import space_out_symbols_in_graph, clean_graph_string, remove_factual_chars


class SceneGraphParser:
    def __init__(self, checkpoint_path, device='cuda:0', lemmatize=False, lowercase=False):
        self.device = device
        self.tokenizer = AutoTokenizer.from_pretrained(checkpoint_path)
        self.model = AutoModelForSeq2SeqLM.from_pretrained(checkpoint_path).to(device).eval()
        self.lemmatize = lemmatize
        self.lowercase = lowercase

        if lemmatize:
            self.lemmatizer = WordNetLemmatizer()

    def _process_text(self, text):
        """
        Perform text processing: lemmatization and optionally converting to lowercase.

        :param text: A string containing the text to be processed.
        :return: Processed text as a string.
        """

        if self.lemmatize:
            # Lemmatize each word in the text
            tokens = text.split(' ')
            text = ' '.join([self.lemmatizer.lemmatize(token) for token in tokens])



        if self.lowercase:
            text = text.lower()

        return text



    def parse(self, descriptions, max_input_len=64, max_output_len=128, beam_size=5, return_text=False, filter_factual_chars=False, batch_size=32):
        if isinstance(descriptions, str):
            descriptions = [descriptions]

        # Apply text processing (lemmatization and lowercase) to each description
        processed_descriptions = [self._process_text(desc) for desc in descriptions]

        # Initialize results list
        all_formatted_texts = []

        # Process descriptions in batches
        for i in tqdm(range(0, len(processed_descriptions), batch_size)):
            batch_descriptions = processed_descriptions[i:i + batch_size]
            prompt_texts = ['Generate Scene Graph: ' + desc.strip() for desc in batch_descriptions]
            with torch.no_grad():
                encoded_inputs = self.tokenizer(
                    prompt_texts,
                    max_length=max_input_len,
                    truncation=True,
                    padding=True,
                    return_tensors='pt'
                )
                tokens = encoded_inputs['input_ids'].to(self.device)
                attention_masks = encoded_inputs['attention_mask'].to(self.device)

                early_stopping = beam_size > 1

                generated_ids = self.model.generate(
                    tokens,
                    attention_mask=attention_masks,
                    use_cache=True,
                    decoder_start_token_id=self.tokenizer.pad_token_id,
                    num_beams=beam_size,
                    max_length=max_output_len,
                    early_stopping=early_stopping,
                    num_return_sequences=1,
                )

                # Decoding the output
                generated_texts = self.tokenizer.batch_decode(generated_ids, skip_special_tokens=True,
                                                              clean_up_tokenization_spaces=True)

                if filter_factual_chars:
                    generated_texts = [remove_factual_chars(text) for text in generated_texts]

                formatted_texts = [clean_graph_string(
                    space_out_symbols_in_graph(text.replace('Generate Scene Graph:', '').strip())) for text in
                                   generated_texts]
                all_formatted_texts.extend(formatted_texts)

        if return_text:
            return all_formatted_texts
        else:
            return [self.graph_string_to_object(text) for text in all_formatted_texts]

    def graph_string_to_object(self, graph_text):
        graph = {'entities': [], 'relations': []}
        entity_map = {}  # Entity name to index mapping

        # Process each relation in the description
        relation_strs = graph_text.strip().split(') ,')
        for relation_str in relation_strs:
            relation_str = relation_str.strip().strip('()')
            parts = [part.strip() for part in relation_str.split(',')]

            if len(parts) != 3 and len(relation_strs) > 1:
                continue  # Skip malformed relations
            elif len(parts) != 3 and len(relation_strs) == 1:
                self._get_or_create_entity_index(parts[0], graph, entity_map)
            else:
                subject, relationship, object_ = parts

                subject_index = self._get_or_create_entity_index(subject, graph, entity_map)

                if relationship == 'is':
                    if object_.isdigit():  # Quantity
                        graph['entities'][subject_index]['quantity'] = object_
                    else:  # Attribute
                        graph['entities'][subject_index]['attributes'].add(object_)
                else:
                    object_index = self._get_or_create_entity_index(object_, graph, entity_map)
                    # Add relation
                    graph['relations'].append({'subject': subject_index, 'relation': relationship, 'object': object_index})

        return graph

    def _get_or_create_entity_index(self, entity_name, graph, entity_map):
        if entity_name not in entity_map:
            new_index = len(graph['entities'])
            graph['entities'].append({'head': entity_name, 'quantity': '', 'attributes': set()})
            entity_map[entity_name] = new_index
        else:
            new_index = entity_map[entity_name]

        return new_index


# unit test main





from setuptools import setup, find_packages

setup(
    name='FactualSceneGraph',
    version='0.5.2',
    author='Zhuang Li',
    author_email='lizhuang144@gmail.com',
    description='A package for scene graph parsing and evaluation',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/zhuang-li/FACTUAL',
    package_dir={'': "src"},
    packages=find_packages(where='src'),
    include_package_data=True,
    package_data={
        'factual_scene_graph.evaluation': ['resources/*'],
                  },
    install_requires=[
        'torch',
        'transformers',
        'tqdm',
        'nltk',
        'spacy',
        'sentence-transformers',
        'pandas',
        'numpy',
        'tabulate'
        # Add other dependencies needed for your package
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Add additional classifiers as appropriate for your project
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
    ],
)



import logging

from factual_scene_graph.evaluation.spice_evaluation import calculate_spice_score, eval_spice
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

if __name__ == "__main__":
    cand = [
        ({"jean"}, {"be"}, {"blue"}),
        [{"jean"}],
        [{"blue"}],
        ({"man"}, {"wear"}, {"jeans"}),
        [{"man"}],
        [{"jeans"}]
    ]

    refs = [
        ({"man"}, {"wear"}, {"jean"}),
        [{"man"}],
        [{"jean"}],
        ({"jean"}, {"be"}, {"blue"}),
        [{"blue"}]
    ]
    print(calculate_spice_score(cand, refs, synonym_match=True))

    cand_graph = '( jean , is , blue ) , ( jean , is , blue )'

    ref_graphs = ['( jean , is , blue ) , ( jeans , is , blue )', '( jean , is , blue ) , ( sea , is , blue ) , ( ocean , is , blue )']

    print(eval_spice(cand_graph, ref_graphs, merge_tuples_synonyms=True, synonym_match=True))

